import inspect
import base64
import sys
import os

nvdblibrary = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe() )))
nvdblibrary = nvdblibrary.replace('\\', '/')
nvdblibrary = nvdblibrary + '/nvdbapi'

## Hvis vi ikke klarer å importere nvdbapiv3 så prøver vi å føye
## mappen nvdblibrary til søkestien.
try:
    import nvdbapiv3
except ModuleNotFoundError:
    print( "Fant ikke nvdbapiv3 i sys.path, legger til mappen", nvdblibrary)
    sys.path.append( nvdblibrary )

    try:
        import nvdbapiv3
    except ModuleNotFoundError as e:
        print( "\nImport av nvdbapiv3 feiler for", nvdblibrary  )
        raise ModuleNotFoundError( "==> Variabel nvdblibrary skal peke til mappen https://github.com/LtGlahn/nvdbapi-V3  <==" )

    else:
        print( "SUKSESS - kan importere nvdbapiv3 etter at vi la til", nvdblibrary, "i sys.path" )
else:
    print( "HURRA - vi kan importere nvdbapiv3 " )

#Qt5 imports
from PyQt5.QtCore import pyqtSignal, QDate

#import need it
from nvdbapiv3 import nvdbFagdata, nvdbVegnett
from .nvdbLesWrapper import AreaGeoDataParser
from .tokenManager import TokenManager
from .vegrefStedfestning import VegrefStedfestning

#pip installed imports
from dotenv import load_dotenv
import xmlschema
import pytest

#function for string encryption/decryption
def enconde_decode(secret: str, is_encoded: bool, byte_on: bool):
    
    if not is_encoded:
        
        sec_byte = base64.b64encode(secret) if byte_on == True else str(base64.b64encode(secret.encode('utf-8')), 'utf-8')
        
        return sec_byte
    
    sec = base64.b64decode(secret) if byte_on == True else str(base64.b64decode(secret), 'utf-8')
        
    return sec

#fixtures for testing NVDB Plug-In
@pytest.fixture
def get_env() -> dict:
    
    dotenvpath = os.path.join(os.path.dirname(__file__), '.env')

    load_dotenv( dotenvpath )

    env = {
        'write_to_nvdb': 'https://nvdbapiskriv.test.atlas.vegvesen.no/rest/v3/endringssett',
        'auth_to_nvdb': 'https://nvdbapiskriv.test.atlas.vegvesen.no/rest/v1/oidc/authenticate',
        'pa': os.environ['localenv_svv_pass'].encode('utf-8'), #https://www.base64encode.org/ is empty, your svv on 64 base encode pass goes here
        'username': os.environ.get('USERNAME'),
        'fylker': { 'fylke': [11, 46]},
        'kommuner': { 'kommune': [1106, 4612]},
        'vegsystemreferanser': { 'vegsystemreferanse': ['ev', 'pv']}
    }
    
    return env

#AreaGeoDataParser class instance
@pytest.fixture
def nvdbles_wrapper_instance() -> None:
    AreaGeoDataParser.set_env('prod')
    
    return AreaGeoDataParser

#nvdb data fagdata
@pytest.fixture #nvdb data for #belysningsstrkning vegobjekt type
def nvdb_data(get_env) -> list:
    
    source_data = nvdbFagdata(86) #Belysningsstrekning
            
    source_data.miljo('test') #test miljø for collecting nvdb data

    source_data.filter( get_env.get('fylker') ) #Rogaland and Vestland fylker
            
    source_data.filter( get_env.get('kommuner') ) #Haugesund and Sveio kommuner
            
    source_data.filter( get_env.get('vegsystemreferanser') ) #europa og privat veg
    
    return source_data.to_records()

@pytest.fixture
def validate_egenskaper_endringssett() -> bool:
    def validate(xml_form):
        
        try:
            schema_base_path = "schemas/"
                
            schema_file_full = schema_base_path + "egenskaper_endringsett_xml_schema.xsd"
            
            schema = xmlschema.XMLSchema(schema_file_full)
                
            with open(schema_base_path + 'current_endringssett.xml', 'w') as file:
                    
                file.write(xml_form.decode('utf-8'))

            schema.validate(schema_base_path + 'current_endringssett.xml')
                
            #removing file not longer need it
            if os.path.exists(schema_base_path + 'current_endringssett.xml'):
                    
                os.remove(schema_base_path + 'current_endringssett.xml')
            
        except Exception as ex:
                
            return False
        
        return True
    
    return validate

@pytest.fixture
def validate_ny_forelder_endringssett() -> bool:
    def validate(xml_form):
        
        try:
            schema_base_path = "schemas/"
                
            schema_file_full = schema_base_path + "ny_forelder_endringsett_xml_schema.xsd"
            
            schema = xmlschema.XMLSchema(schema_file_full)
                
            with open(schema_base_path + 'current_endringssett.xml', 'w') as file:
                    
                file.write(xml_form.decode('utf-8'))

            schema.validate(schema_base_path + 'current_endringssett.xml')
                
            #removing file not longer need it
            if os.path.exists(schema_base_path + 'current_endringssett.xml'):
                    
                os.remove(schema_base_path + 'current_endringssett.xml')
            
        except Exception as ex:
                
            return False
        
        return True
    
    return validate
    
@pytest.fixture
def validate_slett_forelder_endringssett() -> bool:
    def validate(xml_form):
        
        try:
            schema_base_path = "schemas/"
                
            schema_file_full = schema_base_path + "slett_forelder_endringsett_xml_schema.xsd"
            
            schema = xmlschema.XMLSchema(schema_file_full)
                
            with open(schema_base_path + 'current_endringssett.xml', 'w') as file:
                    
                file.write(xml_form.decode('utf-8'))

            schema.validate(schema_base_path + 'current_endringssett.xml')
                
            #removing file not longer need it
            if os.path.exists(schema_base_path + 'current_endringssett.xml'):
                    
                os.remove(schema_base_path + 'current_endringssett.xml')
            
        except Exception as ex:
                
            return False
        
        return True
    
    return validate

@pytest.fixture
def validate_replace_forelder_endringssett() -> bool:
    def validate(xml_form):
        
        try:
            schema_base_path = "schemas/"
                
            schema_file_full = schema_base_path + "replace_forelder_endringsett_xml_schema.xsd"
            
            schema = xmlschema.XMLSchema(schema_file_full)
                
            with open(schema_base_path + 'current_endringssett.xml', 'w') as file:
                    
                file.write(xml_form.decode('utf-8'))

            schema.validate(schema_base_path + 'current_endringssett.xml')
                
            #removing file not longer need it
            if os.path.exists(schema_base_path + 'current_endringssett.xml'):
                    
                os.remove(schema_base_path + 'current_endringssett.xml')
            
        except Exception as ex:
                
            return False
        
        return True
    
    return validate

@pytest.fixture
def validate_stedfest_punkt_endringssett() -> bool:
    def validate(xml_form):
        
        try:
            schema_base_path = "schemas/"
                
            schema_file_full = schema_base_path + "stedfest_punkt_endringsett_xml_schema.xsd"
            
            schema = xmlschema.XMLSchema(schema_file_full)
                
            with open(schema_base_path + 'current_endringssett.xml', 'w') as file:
                    
                file.write(xml_form.decode('utf-8'))

            schema.validate(schema_base_path + 'current_endringssett.xml')
                
            #removing file not longer need it
            if os.path.exists(schema_base_path + 'current_endringssett.xml'):
                    
                os.remove(schema_base_path + 'current_endringssett.xml')
            
        except Exception as ex:
                
            return False
        
        return True
    
    return validate

@pytest.fixture
def validate_stedfest_linje_endringssett() -> bool:
    def validate(xml_form):
        
        try:
            schema_base_path = "schemas/"
                
            schema_file_full = schema_base_path + "stedfest_linje_endringsett_xml_schema.xsd"
            
            schema = xmlschema.XMLSchema(schema_file_full)
                
            with open(schema_base_path + 'current_endringssett.xml', 'w') as file:
                    
                file.write(xml_form.decode('utf-8'))

            schema.validate(schema_base_path + 'current_endringssett.xml')
                
            #removing file not longer need it
            if os.path.exists(schema_base_path + 'current_endringssett.xml'):
                    
                os.remove(schema_base_path + 'current_endringssett.xml')
            
        except Exception as ex:
                
            return False
        
        return True
    
    return validate
#token manager instance for login
@pytest.fixture
def login_instance(get_env) -> dict:

    secret = get_env.get('pa')
        
    auth = enconde_decode(secret, True, False)
        
    tk_manager = TokenManager(os.environ.get('username'), auth, get_env.get('auth_to_nvdb'))
    
    tokens = tk_manager.getToken()

    return tokens

@pytest.fixture
def vegref_instance(login_instance):
    instance = VegrefStedfestning()
    
    instance.set_env('test')
    
    instance.set_attributes({'token': login_instance.get('idToken')})
    
    return instance

@pytest.fixture
def vegobjekt_egenskaper_test(get_env, nvdbles_wrapper_instance) -> dict:
    
    #egenskaper dict and extra_data dict
    
    env_write_endpoint = get_env.get('write_to_nvdb')
    
    datacatalog_version = nvdbles_wrapper_instance.get_datacatalog_version('Akseptansetest')
    
    road_object_type = 86
    
    road_object_name = 'Belysningsstrekning'
    
    nvdbid = 893884859
    
    version = nvdbles_wrapper_instance.get_last_version(nvdbid, road_object_type)
    
    username = get_env.get('username')
    
    #egenskaper
    egenskaper = {
        'nvdbid': nvdbid,
        'versjon': version,
        'Type': 'Bel.pkt en side av veg'
    }
    
    sistmodifisert = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, nvdbid, version)
    
    #extra data
    extra_data = {
        'nvdb_object_type': road_object_type,
        'username': username,
        'datakatalog_version': datacatalog_version,
        'endpoint': env_write_endpoint,
        'sistmodifisert': sistmodifisert,
        'current_nvdbid': nvdbid,
        #'relation': relations,  # dict not need it. Only egenskaper sent
        # 'geometry_found': self.geometry_found,
        'objekt_navn': road_object_name
    }
    
    return {'extra': extra_data, 'egenskaper': egenskaper}

@pytest.fixture
def vegobjekt_sammekobling_ny_forelder_test(get_env, nvdbles_wrapper_instance) -> dict:
    
    #egenskaper dict and extra_data dict
    
    env_write_endpoint = get_env.get('write_to_nvdb')
    
    datacatalog_version = nvdbles_wrapper_instance.get_datacatalog_version('Akseptansetest')
    
    road_object_type = 86
    
    road_object_name = 'Belysningsstrekning'
    
    road_object_possible_child_name = 'Belysningspunkt'
    
    nvdbid = 893884859 #forelder nvdbid
    
    child_nvdbid = 893884873 #belysningspunkt nvdbid
    
    version = nvdbles_wrapper_instance.get_last_version(nvdbid, road_object_type)
    
    username = get_env.get('username')
    
    #egenskaper
    modified_data = {
        'nvdbid': nvdbid,
        'versjon': version
        # 'Type': 'Bel.pkt en side av veg'
    }
    
    last_time_road_object_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, nvdbid, version)
    
    relation = nvdbles_wrapper_instance.get_children_relation_from_parent(road_object_type, nvdbid)
    
    datacatalog_enumid = nvdbles_wrapper_instance.get_datacatalog_relation_type(road_object_type, road_object_possible_child_name)
    
    #extra data
    extra_data = {
        'current_nvdbid': nvdbid,
        'nvdb_object_type': road_object_type,
        'datakatalog_version': datacatalog_version,  # datacatalog current version
        'sistmodifisert': last_time_road_object_modified,
        'username': username,
        'endpoint': env_write_endpoint,
        'objekt_navn': road_object_name,  # test name
        'relation': relation,
        'isChildFoundInParent': False, #up to client code to decide
        'datacatalog_enumId': datacatalog_enumid,
        'add_child_nvdbid': child_nvdbid  # tagging road object to be removed
    }
    
    return {'extra': extra_data, 'modified_data': modified_data}

@pytest.fixture
def vegobjekt_sammekobling_slett_forelder_test(get_env, nvdbles_wrapper_instance) -> dict:
    
    #egenskaper dict and extra_data dict
    
    env_write_endpoint = get_env.get('write_to_nvdb')
    
    datacatalog_version = nvdbles_wrapper_instance.get_datacatalog_version('Akseptansetest')
    
    road_object_type = 86
    
    road_object_name = 'Belysningsstrekning'
    
    road_object_possible_child_name = 'Belysningspunkt'
    
    nvdbid = 893884859 #forelder nvdbid
    
    child_nvdbid = 893884873 #belysningspunkt nvdbid
    
    version = nvdbles_wrapper_instance.get_last_version(nvdbid, road_object_type)
    
    username = get_env.get('username')
    
    #egenskaper
    modified_data = {
        'nvdbid': nvdbid,
        'versjon': version
        # 'Type': 'Bel.pkt en side av veg'
    }
    
    last_time_road_object_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, nvdbid, version)
    
    relation = nvdbles_wrapper_instance.get_children_relation_from_parent(road_object_type, nvdbid)
    
    datacatalog_enumid = nvdbles_wrapper_instance.get_datacatalog_relation_type(road_object_type, road_object_possible_child_name)
    
    if relation:

        # print('has relation')
                
        for datacatalog_id, items in relation.items():
                    
            for nvdbid_ in items['vegobjekter']:
                        
                if nvdbid_ == child_nvdbid:
                    # print('road object to remove found')
                
                    child_in_parent_nvdbid_found = nvdbid_
    
    #extra data
    extra_data = {
        'current_nvdbid': nvdbid,
        'nvdb_object_type': road_object_type,
        'datakatalog_version': datacatalog_version, #datacatalog current version
        'sistmodifisert': last_time_road_object_modified,
        'username': username,
        'endpoint': env_write_endpoint,
        'objekt_navn': road_object_name, #test name
        'relation': relation,
        'remove_child_nvdbid': child_nvdbid if child_in_parent_nvdbid_found == 0 else child_in_parent_nvdbid_found
    }
    
    return {'extra': extra_data, 'modified_data': modified_data}

@pytest.fixture
def vegobjekt_sammekobling_replace_forelder_test(get_env, nvdbles_wrapper_instance) -> dict:
    
    #egenskaper dict and extra_data dict
    
    env_write_endpoint = get_env.get('write_to_nvdb')
    
    datacatalog_version = nvdbles_wrapper_instance.get_datacatalog_version('Akseptansetest')
    
    road_object_type = 86
    
    road_object_name = 'Belysningsstrekning'
    
    # road_object_possible_child_name = 'Belysningspunkt'
    
    current_parent_nvdbid = 893884859 #forelder nvdbid
    
    parent_to_add_nvdbid = 893884813
    
    child_nvdbid = 893884873 #belysningspunkt nvdbid
    
    target_parent_remove = 893884859
    
    version = nvdbles_wrapper_instance.get_last_version(current_parent_nvdbid, road_object_type)
    
    username = get_env.get('username')
    
    #egenskaper
    modified_data = {
        'nvdbid': current_parent_nvdbid,
        'versjon': version
        # 'Type': 'Bel.pkt en side av veg'
    }
    
    last_time_road_object_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, current_parent_nvdbid, version)
    
    # relation = nvdbles_wrapper_instance.get_children_relation_from_parent(road_object_type, current_parent_nvdbid)
    
    # datacatalog_enumid = nvdbles_wrapper_instance.get_datacatalog_relation_type(road_object_type, road_object_possible_child_name)
    
    version_parent_remove = nvdbles_wrapper_instance.get_last_version(target_parent_remove, road_object_type)
    
    parent_remove_last_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, target_parent_remove, version_parent_remove)
    
    parent_add_last_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, parent_to_add_nvdbid, version)
    
    relation_old_parent = nvdbles_wrapper_instance.get_possible_children_types(road_object_type)

    #extra data
    extra_data = {
        'env': 'Akseptansetest',
        'start_date': QDate.currentDate().toString('yyyy-MM-dd'),
        'endringsett_mode': 'correction', #'update', #'correction',
        'add_to_parent': parent_to_add_nvdbid, #parent_nvdbid,
        'current_nvdbid': current_parent_nvdbid,
        'version_old_parent_type': road_object_type,
        'version_new_parent_type': road_object_type,
        # 'nvdb_object_type_add_parent': self.parent_type_selected_to_relate,
        'datakatalog_version': datacatalog_version,  # datacatalog current version
        'sistmodifisert_add': parent_add_last_modified,
        'sistmodifisert_remove': parent_remove_last_modified,
        'username': username,
        'endpoint': env_write_endpoint,
        'objekt_navn': road_object_name,  # test name
        'relation_old_parent': relation_old_parent,
        'relation_new_parent': relation_old_parent, #can use same, because is same type in the test fixture
        'child_nvdbid': child_nvdbid,
        'remove_from_parent': target_parent_remove, #self.parent_roadObject_linked_nvdbid[1]
        'road_object_name': 'Belysningspunkt' # name to barn vegobjekt
    }
    
    return {'extra': extra_data, 'modified_data': modified_data}
    

@pytest.fixture
def vegobjekt_stedfest_punkt_test(get_env, nvdbles_wrapper_instance, vegref_instance) -> dict:
    
    #egenskaper dict and extra_data dict
    
    env_write_endpoint = get_env.get('write_to_nvdb')
    
    datacatalog_version = nvdbles_wrapper_instance.get_datacatalog_version('Akseptansetest')
    
    road_object_type = 87
    
    road_object_name = 'Belysningspunkt'
    
    nvdbid = 893884873 #forelder nvdbid
        
    version = nvdbles_wrapper_instance.get_last_version(nvdbid, road_object_type)
    
    username = get_env.get('username')
    
    last_time_road_object_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, nvdbid, version)
    
    relation = nvdbles_wrapper_instance.get_children_relation_from_parent(road_object_type, nvdbid)
    
    datacatalog_enumid = nvdbles_wrapper_instance.get_datacatalog_relation_type(road_object_type, road_object_name)
    
    stedfest_type = nvdbles_wrapper_instance.get_geometry_type_by_nvdbid(nvdbid, road_object_type)
    
    object_sideposition = nvdbles_wrapper_instance.get_parent_sideposisjon_og(road_object_type, nvdbid)
    
    # start_dato = '2025-01-30T10:44:11'
    
    geometry = 'Point (-46864.98381967330351472 6625553.24016814772039652)'
    
    #setting geometry data to calculate vegref instance
    attribs = {
        'type': road_object_type,
        'nvdbid': nvdbid,
        'startdato': last_time_road_object_modified,
        'geometry': geometry,
        'stedfest_type': 'punkt',
        'env': 'test'
        }
    
    v = vegref_instance.get_vegref_from_wkt(attribs)
    
    if vegref_instance.check_vegref(v.get('from')):
        
        #vegref is valid
        extra_vegref_data = {
            'start_date': last_time_road_object_modified,
            'trafikant_gruppe': 'k',
            'nvdbid': nvdbid,
            'type': road_object_type,
            'versjon': version
        }
        
        attribs_vegref_data = {
            'stedfest_type': 'punkt',
            'fra': v.get('from')
        }
        
        vegref_instance.get_position(attribs_vegref_data)
                    
        #stedfestning road object
        vegref_instance.stedfest_object( extra_vegref_data )
    
    geo_data = {
        'location': vegref_instance.egenskaper_stedfest,
        'position': vegref_instance.geo_parsed
    }

    # modified data
    modified_data = {
        'nvdbid': nvdbid,
        'versjon': version  # last version to road object
    }

    # extra data need it for xml escheme completion
    extra_data = {
        'current_nvdbid': nvdbid,
        'nvdb_object_type': road_object_type,
        'datakatalog_version': datacatalog_version,  # datacatalog current version
        'sistmodifisert': last_time_road_object_modified,
        'username': username,
        'endpoint': env_write_endpoint,
        'objekt_navn': road_object_name,  # test name
        'datacatalog_enumId': datacatalog_enumid,
        'stedfest': geo_data, 
        'stedfest_type': stedfest_type,  # for now
        'object_sideposisjon': object_sideposition,
        'is_child_included': False,
        'has_relation': True if len( relation ) > 0 else False,
        'relation': relation
    }
    
    return {'extra': extra_data, 'modified_data': modified_data}
    

@pytest.fixture
def vegobjekt_stedfest_linje_test(get_env, nvdbles_wrapper_instance, vegref_instance) -> dict:
    
    #egenskaper dict and extra_data dict
    
    env_write_endpoint = get_env.get('write_to_nvdb')
    
    datacatalog_version = nvdbles_wrapper_instance.get_datacatalog_version('Akseptansetest')
    
    road_object_type = 86
    
    road_object_name = 'Belysningsstrekning'
    
    nvdbid = 893884813 #forelder nvdbid
        
    version = nvdbles_wrapper_instance.get_last_version(nvdbid, road_object_type)
    
    username = get_env.get('username')
    
    last_time_road_object_modified = nvdbles_wrapper_instance.get_last_time_modified(road_object_type, nvdbid, version)
    
    relation = nvdbles_wrapper_instance.get_children_relation_from_parent(road_object_type, nvdbid)
    
    datacatalog_enumid = nvdbles_wrapper_instance.get_datacatalog_relation_type(road_object_type, road_object_name)
    
    stedfest_type = nvdbles_wrapper_instance.get_geometry_type_by_nvdbid(nvdbid, road_object_type)
    
    object_sideposition = nvdbles_wrapper_instance.get_parent_sideposisjon_og(road_object_type, nvdbid)
    
    # start_dato = '2025-01-31T13:03:17'
    
    geometry = 'LINESTRING Z(-47550.657 6624984.849 51.405, -47526.372 6625035.689 50.369)'
    
    #setting geometry data to calculate vegref instance
    attribs = {
        'type': road_object_type,
        'nvdbid': nvdbid,
        'startdato': last_time_road_object_modified,
        'geometry': geometry,
        'stedfest_type': 'linje',
        'env': 'test'
        }
    
    v = vegref_instance.get_vegref_from_wkt(attribs)
    
    if vegref_instance.check_vegref(v.get('from')) and vegref_instance.check_vegref(v.get('to')):
        
        #vegref is valid
        extra_vegref_data = {
            'start_date': last_time_road_object_modified,
            'trafikant_gruppe': 'k',
            'nvdbid': nvdbid,
            'type': road_object_type,
            'versjon': version
        }
        
        attribs_vegref_data = {
            'stedfest_type': 'linje',
            'fra': v.get('from'),
            'til': v.get('to')
        }
        
        vegref_instance.get_position(attribs_vegref_data)
                    
        #stedfestning road object
        vegref_instance.stedfest_object( extra_vegref_data )
    
    geo_data = {
        'location': vegref_instance.egenskaper_stedfest,
        'position': vegref_instance.geo_parsed
    }

    # modified data
    modified_data = {
        'nvdbid': nvdbid,
        'versjon': version  # last version to road object
    }

    # extra data need it for xml escheme completion
    extra_data = {
        'current_nvdbid': nvdbid,
        'nvdb_object_type': road_object_type,
        'datakatalog_version': datacatalog_version,  # datacatalog current version
        'sistmodifisert': last_time_road_object_modified,
        'username': username,
        'endpoint': env_write_endpoint,
        'objekt_navn': road_object_name,  # test name
        'datacatalog_enumId': datacatalog_enumid,
        'stedfest': geo_data, 
        'stedfest_type': stedfest_type,  # for now
        'object_sideposisjon': object_sideposition,
        'is_child_included': False,
        'has_relation': True if len( relation ) > 0 else False,
        'relation': relation
    }
    
    return {'extra': extra_data, 'modified_data': modified_data}