########### user defined imports ###########################################
from .delvisKorrEgenskaperCase import DelvisKorrEgenskaperCase #egenskaper endringssett
from .customKorrSingleAdd import CustomDelvisKorrSingleAdd
from .customDelvisKorrRemoveCase import CustomDelvisKorrRemoveCase
from .customDelvisKorrReplaceParent import CustomDelvisKorrReplaceParent
from .delvisStedfestCase import DelvisStedfestCase

from .config_t import *

#PyQGIS imports
from PyQt5.QtCore import pyqtSignal

################### tests ########################
def test_env_conf(get_env):
    
    assert not get_env.get('pa') == ''
    assert not get_env.get('write_to_nvdb') == ''
    assert not get_env.get('auth_to_nvdb') == ''
    assert not get_env.get('username') == ''
    assert not get_env.get('fylker') == ''
    assert not get_env.get('kommuner') == ''
    assert not get_env.get('vegsystemreferanser') == ''
    
def test_laste_ned_fylker(nvdbles_wrapper_instance):
    
    fylker = nvdbles_wrapper_instance.counties()
    
    if len( fylker ) == 0:
        
        pass
        
    assert len(fylker) == 15

def test_laste_ned_kommuner(nvdbles_wrapper_instance):
    
    kommuner = nvdbles_wrapper_instance.communities()
    
    if len( kommuner ) == 0:
        
        pass
        
    assert len(kommuner) == 355

def test_laste_ned_datakatalog(nvdbles_wrapper_instance):
    
    datakatalog = nvdbles_wrapper_instance.fetchAllNvdbObjects()
    
    if len( datakatalog ) == 0:
        
        pass
        
    assert len(datakatalog) == 417

def test_laste_ned_egenskaper(nvdbles_wrapper_instance):
    
    type_id = nvdbles_wrapper_instance.fetchAllNvdbObjects()['Veganlegg']
    
    egenskaper = nvdbles_wrapper_instance.egenskaper( type_id )
    
    if len( egenskaper ) == 0:
        
        pass
    
    assert len(egenskaper) > 0

def test_laste_ned_nvdb_data(nvdb_data, get_env):
    
    if len( nvdb_data ) == 0:
        
        pass
        
    assert len(nvdb_data) > 0

def test_login_nvdb_skriv(login_instance, get_env):
    
    if len( login_instance.get('idToken') ) == 0:
        
        pass
        
    assert login_instance.get('idToken') != ''

def test_preparing_endringssett_egenskaper(login_instance, vegobjekt_egenskaper_test, nvdbles_wrapper_instance, validate_egenskaper_endringssett):
    
    token = login_instance.get('idToken')
    
    type_id = nvdbles_wrapper_instance.fetchAllNvdbObjects()['Belysningsstrekning']
    
    egenskaper = nvdbles_wrapper_instance.egenskaper( type_id )
    
    delvis = DelvisKorrEgenskaperCase(token, vegobjekt_egenskaper_test.get('egenskaper'), vegobjekt_egenskaper_test.get('extra'))
    
    delvis.formXMLRequest(egenskaper)
        
    val = validate_egenskaper_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("egenskaper endringssett ikke valid", allow_module_level=True)
        
    assert val == True


def test_kan_send_endringssett_egenskaper(login_instance, vegobjekt_egenskaper_test, nvdbles_wrapper_instance, validate_egenskaper_endringssett):
    
    token = login_instance.get('idToken')
    
    type_id = nvdbles_wrapper_instance.fetchAllNvdbObjects()['Belysningsstrekning']
    
    egenskaper = nvdbles_wrapper_instance.egenskaper( type_id )
    
    delvis = DelvisKorrEgenskaperCase(token, vegobjekt_egenskaper_test.get('egenskaper'), vegobjekt_egenskaper_test.get('extra'))

    delvis.endringsett_form_done.connect(delvis.prepare_post)

    delvis.formXMLRequest(egenskaper)
    
    val = validate_egenskaper_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("endringssett kan ikke sent, pga endringsett er ikke valid!", allow_module_level=True)
    
    print(delvis.list_vegobjekter_info.get('status_after_sent'))
    
    assert not delvis.list_vegobjekter_info.get('status_after_sent') == ' '

def test_preparing_endringssett_ny_forelder(login_instance, vegobjekt_sammekobling_ny_forelder_test, nvdbles_wrapper_instance, validate_ny_forelder_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_sammekobling_ny_forelder_test.get('modified_data')
    
    extra = vegobjekt_sammekobling_ny_forelder_test.get('extra')
    
    delvis = CustomDelvisKorrSingleAdd(token, modified_data, extra)

    delvis.formXMLRequest()
    
    val = validate_ny_forelder_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("ny forelder endringssett ikke valid", allow_module_level=True)
    
    assert len(delvis.xml_string) > 0

def test_kan_send_endringssett_ny_forelder(login_instance, vegobjekt_sammekobling_ny_forelder_test, nvdbles_wrapper_instance, validate_ny_forelder_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_sammekobling_ny_forelder_test.get('modified_data')
    
    extra = vegobjekt_sammekobling_ny_forelder_test.get('extra')
    
    delvis = CustomDelvisKorrSingleAdd(token, modified_data, extra)
    
    delvis.endringsett_form_done.connect(delvis.prepare_post)

    delvis.formXMLRequest()
    
    val = validate_ny_forelder_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("kan ikke sent ny forelder endringssett pga endringssett er ikke valid", allow_module_level=True)
    
    print(delvis.list_vegobjekter_info.get('status_after_sent'))
    
    assert not delvis.list_vegobjekter_info.get('status_after_sent') == ' '

def test_preparing_endringssett_slett_forelder(login_instance, vegobjekt_sammekobling_slett_forelder_test, nvdbles_wrapper_instance, validate_slett_forelder_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_sammekobling_slett_forelder_test.get('modified_data')
    
    extra = vegobjekt_sammekobling_slett_forelder_test.get('extra')
    
    delvis = CustomDelvisKorrRemoveCase(token, modified_data, extra)
    
    delvis.formXMLRequest()
    
    val = validate_slett_forelder_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("slett forelder endringssett er ikke valid", allow_module_level=True)
    
    assert len(delvis.xml_string) > 0


def test_kan_send_endringssett_slett_forelder(login_instance, vegobjekt_sammekobling_slett_forelder_test, nvdbles_wrapper_instance, validate_slett_forelder_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_sammekobling_slett_forelder_test.get('modified_data')
    
    extra = vegobjekt_sammekobling_slett_forelder_test.get('extra')
    
    delvis = CustomDelvisKorrRemoveCase(token, modified_data, extra)
    
    delvis.endringsett_form_done.connect(delvis.prepare_post)

    delvis.formXMLRequest()
    
    val = validate_slett_forelder_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("kan ikke sent slett forelder endringssett pga endringssett er ikke valid", allow_module_level=True)
    
    print(delvis.list_vegobjekter_info.get('status_after_sent'))
    
    assert not delvis.list_vegobjekter_info.get('status_after_sent') == ' '
    
def test_preparing_endringssett_replace_forelder(login_instance, vegobjekt_sammekobling_replace_forelder_test, nvdbles_wrapper_instance, validate_replace_forelder_endringssett):
    
    token = login_instance.get('idToken')
    
    # modified_data = vegobjekt_sammekobling_replace_forelder_test.get('modified_data') # not need for replace endringssett
    
    extra = vegobjekt_sammekobling_replace_forelder_test.get('extra')
    
    delvis = CustomDelvisKorrReplaceParent(token, {}, extra)
    
    delvis.formXMLRequest()
    
    val = validate_replace_forelder_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("replace forelder endringssett er ikke valid", allow_module_level=True)
    
    assert len(delvis.xml_string) > 0

def test_kan_send_endringssett_replace_forelder(login_instance, vegobjekt_sammekobling_replace_forelder_test, nvdbles_wrapper_instance, validate_replace_forelder_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_sammekobling_replace_forelder_test.get('modified_data')
    
    extra = vegobjekt_sammekobling_replace_forelder_test.get('extra')
    
    delvis = CustomDelvisKorrReplaceParent(token, {}, extra)
    
    delvis.endringsett_form_done.connect(delvis.prepare_post)

    delvis.formXMLRequest()
    
    val = validate_replace_forelder_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("kan ikke sent slett forelder endringssett pga endringssett er ikke valid", allow_module_level=True)
    
    print(delvis.list_vegobjekter_info.get('status_after_sent'))
    
    assert not delvis.list_vegobjekter_info.get('status_after_sent') == ' '

def test_preparing_endringssett_stedfest_punkt(login_instance, vegobjekt_stedfest_punkt_test, nvdbles_wrapper_instance, validate_stedfest_punkt_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_stedfest_punkt_test.get('modified_data') # not need for replace endringssett
    
    extra = vegobjekt_stedfest_punkt_test.get('extra')
    
    delvis = DelvisStedfestCase(token, modified_data, extra)
    
    delvis.formXMLRequest([])
    
    val = validate_stedfest_punkt_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("stedfest punkt endringssett er ikke valid", allow_module_level=True)
    
    assert len(delvis.xml_string) > 0

def test_kan_send_endringssett_stedfest_punkt(login_instance, vegobjekt_stedfest_punkt_test, nvdbles_wrapper_instance, validate_stedfest_punkt_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_stedfest_punkt_test.get('modified_data')
    
    extra = vegobjekt_stedfest_punkt_test.get('extra')
    
    delvis = DelvisStedfestCase(token, modified_data, extra)
    
    delvis.endringsett_form_done.connect(delvis.prepare_post)

    delvis.formXMLRequest([])
    
    val = validate_stedfest_punkt_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("kan ikke sent stedfest punkt endringssett pga endringssett er ikke valid", allow_module_level=True)
    
    print(delvis.list_vegobjekter_info.get('status_after_sent'))
    
    assert not delvis.list_vegobjekter_info.get('status_after_sent') == ' '

def test_preparing_endringssett_stedfest_linje(login_instance, vegobjekt_stedfest_linje_test, nvdbles_wrapper_instance, validate_stedfest_linje_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_stedfest_linje_test.get('modified_data') # not need for replace endringssett
    
    extra = vegobjekt_stedfest_linje_test.get('extra')
    
    delvis = DelvisStedfestCase(token, modified_data, extra)
    
    delvis.formXMLRequest([])
    
    val = validate_stedfest_linje_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip(f"stedfest linje endringssett er ikke valid", allow_module_level=True)
    
    assert len(delvis.xml_string) > 0

def test_kan_send_endringssett_stedfest_linje(login_instance, vegobjekt_stedfest_linje_test, nvdbles_wrapper_instance, validate_stedfest_linje_endringssett):
    
    token = login_instance.get('idToken')
    
    modified_data = vegobjekt_stedfest_linje_test.get('modified_data')
    
    extra = vegobjekt_stedfest_linje_test.get('extra')
    
    delvis = DelvisStedfestCase(token, modified_data, extra)
    
    delvis.endringsett_form_done.connect(delvis.prepare_post)

    delvis.formXMLRequest([])
    
    val = validate_stedfest_linje_endringssett(delvis.xml_string)
    
    if not val:
        
        pytest.skip("kan ikke sent stedfest punkt endringssett pga endringssett er ikke valid", allow_module_level=True)
    
    print(delvis.list_vegobjekter_info.get('status_after_sent'))
    
    assert not delvis.list_vegobjekter_info.get('status_after_sent') == ' '
